@@ -3,38 +3,35 @@
 from decimal import Decimal
 
 from trytond.model import fields
-from trytond.pool import PoolMeta, Pool
-from trytond.pyson import Eval
-
+from trytond.modules.currency.fields import Monetary
 from trytond.modules.product import price_digits, round_price
+from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Eval
 
 
 class Line(metaclass=PoolMeta):
     __name__ = 'sale.line'
 
-    base_price = fields.Numeric(
-        "Base Price", digits=price_digits,
+    base_price = Monetary(
+        "Base Price", currency='currency', digits=price_digits,
         states={
             'invisible': Eval('type') != 'line',
             'readonly': Eval('sale_state') != 'draft',
-            },
-        depends=['type', 'sale_state'])
+            })
 
     discount_rate = fields.Function(fields.Numeric(
             "Discount Rate", digits=(16, 4),
             states={
                 'invisible': Eval('type') != 'line',
                 'readonly': Eval('sale_state') != 'draft',
-                },
-            depends=['type', 'sale_state']),
+                }),
         'on_change_with_discount_rate', setter='set_discount_rate')
-    discount_amount = fields.Function(fields.Numeric(
-            "Discount Amount", digits=price_digits,
+    discount_amount = fields.Function(Monetary(
+            "Discount Amount", currency='currency', digits=price_digits,
             states={
                 'invisible': Eval('type') != 'line',
                 'readonly': Eval('sale_state') != 'draft',
-                },
-            depends=['type', 'sale_state']),
+                }),
         'on_change_with_discount_amount', setter='set_discount_amount')
 
     discount = fields.Function(fields.Char(
@@ -60,7 +57,7 @@ class Line(metaclass=PoolMeta):
     def compute_base_price(self):
         pool = Pool()
         Uom = pool.get('product.uom')
-        if self.product:
+        if self.product and self.product.list_price is not None:
             price = self.product.list_price
             if self.unit:
                 price = Uom.compute_price(
@@ -140,8 +137,8 @@ class Line(metaclass=PoolMeta):
                     return lang.currency(
                         amount, self.sale.currency, digits=price_digits[1])
                 else:
-                    return lang.format(
-                        '%.f', (price_digits[1], amount), monetary=True)
+                    return lang.format_number(
+                        amount, digits=price_digits[1], monetary=True)
         else:
             return lang.format('%i', rate * 100) + '%'
 
